# coding: utf-8

from .base import BasePattern


class CyclePattern(BasePattern):
    """
    select an element regularly from the choices.
    """
    def __init__(self, choices, increment=1, callback=None):
        """
        :param choices: list-elements of choices
        :param increment: list-index increase.
        :param callback: function to be called at the end of rendering.
        :return: Cycle Pattern instance.
        """
        if isinstance(choices, dict):
            self.choices = []
            for value, num in choices.items():
                self.choices += [value] * num
        else:
            self.choices = choices
        self.size = len(self.choices)
        self.index = 0
        self.increment = increment
        self.add_callback(callback)

    def render(self, *args):
        """
        render a element by regularly selection
        :param args: any.
        :return: rendered element.
        """
        element = self.choices[self.index % self.size]
        self.index += self.increment
        for callback in self.callbacks:
            element = callback(element)
        return element
